@@ -2,12 +2,12 @@ import json
 from abc import ABC, abstractclassmethod
 from dataclasses import asdict
 from pathlib import Path
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, Iterable, List, Union
 
 import pandas as pd
 
 from digital_experiments.experiment import Experiment
-from digital_experiments.util import flatten, pretty_json
+from digital_experiments.util import flatten, pretty_json, unflatten
 
 
 class Files:
@@ -21,11 +21,9 @@ class Backend(ABC):
 
     A backend is responsible for saving and loading experiments
     Implementations should subclass this class and implement the abstract methods
-    Any core files (e.g. config.json) should be in the `core_files` class attribute
+    Any core files (e.g. config.json) should be returned by the `core_files`
     """
 
-    core_files = []
-
     @abstractclassmethod
     def save(
         cls,
@@ -37,25 +35,23 @@ class Backend(ABC):
         pass
 
     @abstractclassmethod
-    def load_all_experiments(cls, root: Path) -> Union[List[Experiment], pd.DataFrame]:
+    def _load_all_experiments(cls, root: Path) -> Iterable[Experiment]:
         pass
 
     @classmethod
-    def all_experiments(cls, root: Path, metadata: bool) -> pd.DataFrame:
-        experiments = cls.load_all_experiments(root)
-        if len(experiments) == 0:
-            return pd.DataFrame()
+    def core_files(self):
+        return []
 
-        if not isinstance(experiments, pd.DataFrame):
-            experiments = pd.DataFrame([flatten(asdict(e)) for e in experiments])
-
-        experiments.sort_values("id", inplace=True)
-        experiments.reset_index(drop=True, inplace=True)
-        return experiments if metadata else experiments.filter(regex="^(?!metadata)")
+    @classmethod
+    def all_experiments(cls, root: Path) -> List[Experiment]:
+        experiments = cls._load_all_experiments(root)
+        return sorted(experiments, key=lambda e: e.metadata["timing"]["start"])
 
 
 class JSONBackend(Backend):
-    core_files = ["config.json", "results.json", "metadata.json"]
+    @classmethod
+    def core_files(cls):
+        return ["config.json", "results.json", "metadata.json"]
 
     @classmethod
     def save(
@@ -70,12 +66,12 @@ class JSONBackend(Backend):
         (exmpt_dir / "metadata.json").write_text(pretty_json(metadata))
 
     @classmethod
-    def load_all_experiments(cls, root) -> List[Experiment]:
+    def _load_all_experiments(cls, root) -> List[Experiment]:
         def experiment_from(dir: Path):
             return Experiment(
                 id=dir.name,
                 config=json.loads((dir / "config.json").read_text()),
-                results=json.loads((dir / "results.json").read_text()),
+                result=json.loads((dir / "results.json").read_text()),
                 metadata=json.loads((dir / "metadata.json").read_text()),
             )
 
@@ -94,14 +90,11 @@ class CSVBackend(Backend):
         file = exmpt_dir.parent / "results.csv"
         previous_experiments = pd.read_csv(file) if file.exists() else pd.DataFrame()
 
-        if not isinstance(result, dict):
-            result = {"results": result}
-
         entry = flatten(
             {
                 "id": exmpt_dir.name,
                 "config": config,
-                "results": result,
+                "result": result,
                 "metadata": metadata,
             }
         )
@@ -109,8 +102,10 @@ class CSVBackend(Backend):
         df.to_csv(file, index=False)
 
     @classmethod
-    def load_all_experiments(cls, root) -> pd.DataFrame:
-        return pd.read_csv(root / "results.csv")
+    def _load_all_experiments(cls, root) -> pd.DataFrame:
+        df = pd.read_csv(root / "results.csv")
+        rows = [unflatten(row) for _, row in df.iterrows()]
+        return [Experiment(**row) for row in rows]
 
 
 __available_backends = {"json": JSONBackend, "csv": CSVBackend}
