@@ -156,12 +156,57 @@ def pretty_json(thing):
     return json.dumps(thing, indent=4, cls=NpEncoder)
 
 
-independent_random = np.random.RandomState(int(datetime.now().timestamp()))
-
-
 def get_complete_config(func, args, kwargs):
     sig = inspect.signature(func)
     config = sig.bind(*args, **kwargs)
     config.apply_defaults()
     config = config.arguments
     return config
+
+
+def matches(thing, template):
+    """
+    does thing conform to the passed (and optionally nested template?)
+
+    e.g.
+    matches({"a": 1, "b": 2}, template={"a": 1}) == True
+    matches({"a": 1, "b": 2}, template={"a": 1, "c": 3}) == False
+    matches({"a": 1, "b": 2}, template={"a": lambda x: x > 0}) == True
+    matches(
+        {"a": 1, "b": {"c": 2}},
+        template={"b": {"c": lambda x: x%2 == 0}}
+    ) == True
+    """
+
+    for key in set(thing.keys()).union(set(template.keys())):
+        if key not in template:
+            # template doesn't specify what to do with this key
+            continue
+        if key not in thing:
+            # thing doesn't have this required key: doesn't match
+            return False
+        if matches_value(thing[key], template[key]):
+            continue
+        else:
+            # value doesn't match
+            return False
+
+    return True
+
+
+def matches_value(value, template_value):
+    """
+    does value conform to the entry in template_value?
+
+    e.g.
+    matches_value(1, 1) == True
+    matches_value(1, 2) == False
+    matches_value(1, lambda x: x > 0) == True
+    matches_value({"a": 1}, {"a": 1}) == True # calls back into matches
+    """
+
+    if isinstance(value, Mapping):
+        return matches(value, template_value)
+    if callable(template_value):
+        return template_value(value)
+    return value == template_value
